//------------------------------------------------------ 2021-09-13 --------------
// air-soft gun plate Target Systems Ver2
//
// Library
//   TimerOne 
//   TM1637 by Avishay Orpaz V1.2.0
//----------------------------- written by Hiroyuki Iizuka ---- Japan ------------

//----- edit here -----------------------------------
#define WAIT_RANDOM_MIN   20    // 1/10 seconds  20 = 2sec
#define WAIT_RANDOM_MAX   40    // 1/10 seconds  40 = 4sec
#define WAIT_FIXED        30    // 1/10 seconds  30 = 3sec
#define DISPLAY_ALWAYS    
// #define BEEP_MAX_VOLUME    
//---------------------------------------------------

#include <TimerOne.h>
#include <TM1637Display.h>
#include <EEPROM.h>

boolean       time_update   = false;
unsigned long millisecond10 = 0;
boolean       clock_run     = false;
unsigned char wait;
unsigned char mode;

#define MODE_INSTANT  0
#define MODE_FIXED    1
#define MODE_RANDOM   2
#define EEPROM_ADDRESS_MODE 0 

#define TM1637_CLK   4   // pin 4
#define TM1637_DIO   5   // pin 5
#define TM1637_COLON_ON   0b01000000
#define TM1637_COLON_OFF  0b00000000
#define MAX_BRIGHTNESS    0x0F

TM1637Display display(TM1637_CLK, TM1637_DIO);

#define BUZZER 8         // pin 8
#define BUZZER_OFF  0
#define BUZZER_ON   1

#define SENSOR 2         // interrupt 0 = pin2

#define IR      6        // pin 6
#define IR_ON   0

#define SW1     A5       // pin A5
#define SW2     A4       // pin A4
#define SW_ON   0

#define LED_RED     9    // pin 9
#define LED_GREEN  10    // pin 10
#define LED_ON  1
#define LED_OFF 0

//----------------------- timer1 interrupts -----------------------
void TimingISR()
{
  if( clock_run ) {
    time_update = true;
    millisecond10++;
  }
}

//----------------------- piezo sensor interrupts -----------------
void impact()
{
  clock_run = false;
  display.showNumberDecEx( millisecond10, TM1637_COLON_ON, false );  
  digitalWrite( LED_BUILTIN, LED_ON  );
  digitalWrite( LED_GREEN,   LED_OFF );
}

//----------------------- beep ------------------------------------
void beep()
{
#ifdef BEEP_MAX_VOLUME
  digitalWrite( BUZZER, BUZZER_ON );
  delay(500);
  digitalWrite( BUZZER, BUZZER_OFF  );
#else  
  tone(BUZZER, 1000);   // 1kHz
  delay(500);
  noTone(BUZZER);
#endif  
}

//----------- get mode --------------------------------------------
unsigned char get_mode( void )
{
  unsigned char mode;
  
  mode = EEPROM.read( EEPROM_ADDRESS_MODE );
  if( mode >= 3 ) {
    mode = 2;  
  }
  return mode;
}

//----------- set mode --------------------------------------------
void set_mode( unsigned char mode )
{
  EEPROM.write( EEPROM_ADDRESS_MODE, mode );
}

//----------------------- setup -----------------------------------
void setup()
{
  pinMode( IR,     INPUT  );
  pinMode( SW1,    INPUT  );
  pinMode( SENSOR, INPUT  );
  pinMode( LED_BUILTIN, OUTPUT );
  pinMode( LED_RED,     OUTPUT );
  pinMode( LED_GREEN,   OUTPUT );
  pinMode( BUZZER,      OUTPUT );

  mode = get_mode();
  
  digitalWrite( LED_RED, LED_OFF );  
  display.setBrightness( MAX_BRIGHTNESS );
  Timer1.initialize(10000);           // 10ms period
  Timer1.attachInterrupt(TimingISR); 

  display.showNumberDecEx(8888, TM1637_COLON_ON, false);
  digitalWrite( LED_RED,   LED_ON ); 
  digitalWrite( LED_GREEN, LED_ON ); 
  beep();
  display.clear();
  digitalWrite( LED_RED,   LED_OFF ); 
  digitalWrite( LED_GREEN, LED_OFF ); 
  display.showNumberDecEx( mode, TM1637_COLON_OFF, false );
    
  clock_run = false;
  randomSeed( analogRead(0) );

  attachInterrupt(0, impact, RISING );   // 0 = pin2
}

//----------------------- loop ------------------------------------
void loop()
{
  if( time_update ) {
#ifdef DISPLAY_ALWAYS    
    display.showNumberDecEx( millisecond10, TM1637_COLON_ON, false );
#endif    
    time_update = false;
  }

  if( (digitalRead( IR ) == IR_ON) || (digitalRead( SW1 ) == SW_ON) ) {
    clock_run = false;
    millisecond10 = 0;
    display.clear();
    switch( mode ) {
      case MODE_RANDOM:
        digitalWrite( LED_RED,   LED_ON );
        display.showNumberDec( 8888, false );
        wait = random( WAIT_RANDOM_MIN, WAIT_RANDOM_MAX );
        break;
      case MODE_FIXED:
        digitalWrite( LED_RED,   LED_ON );
        display.showNumberDec( 8888, false );   
        wait = WAIT_FIXED;
        break;
      case MODE_INSTANT:
        clock_run = true; 
        digitalWrite( LED_GREEN, LED_ON );   
        display.clear();
        beep();
        delay(100);
        while( digitalRead( SW2 ) == SW_ON ) { ; }
        delay(100);
        break;
    }
    digitalWrite( LED_RED, LED_OFF );
  }

  if( digitalRead( SW2 ) == SW_ON ) {
    switch( mode ) {
      case MODE_INSTANT: mode = MODE_FIXED;   break;
      case MODE_FIXED:   mode = MODE_RANDOM;  break;
      case MODE_RANDOM:  mode = MODE_INSTANT; break;
    }
    set_mode( mode );
    display.clear();
    display.showNumberDec( mode, false );
    delay(50);
    while( digitalRead( SW2 ) == SW_ON ) { ; }
    delay(50);
  }
  
  if( wait != 0 ) {
    while( wait > 0 ) {
      delay(100);
      wait--;
    }
    clock_run = true; 
    digitalWrite( LED_GREEN, LED_ON );   
    display.clear();
    beep();
  }
}

//--------------- E N D -------------------------------------------------------------------
